-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2011.11.06					---
--- Version: v0.9 [2011.11.19]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/fastercamera
--- WoWInterface	http://www.wowinterface.com/downloads/info20483-FasterCamera.html

local VERSION = 0.9
local NAME = "FasterCamera"
local L = LibStub("AceLocale-3.0"):GetLocale("FasterCamera", true)
local db

	----------------
	--- Prehooks ---
	----------------

local oldZoomIn = CameraZoomIn
local oldZoomOut = CameraZoomOut

function CameraZoomIn(distance)
	oldZoomIn(db.increment)
end

function CameraZoomOut(distance)
	oldZoomOut(db.increment)
end

-- tried this out in the SotA demolishers
-- the normal camera zoom functions seemed to be used instead
-- I suppose both can be used interchangeably
local oldVehicleZoomIn = VehicleCameraZoomIn
local oldVehicleZoomOut = VehicleCameraZoomOut

function VehicleCameraZoomIn(distance)
	oldVehicleZoomIn(db.increment)
end

function VehicleCameraZoomOut(distance)
	oldVehicleZoomOut(db.increment)
end

	---------------
	--- Options ---
	---------------

local cvar = {
	"cameraDistanceMoveSpeed",
	"cameraDistanceMax",
	"cameraDistanceMaxFactor",
}

local options = {
	type = "group",
	name = NAME.." |cffADFF2Fv"..VERSION.."|r",
	args = {
		inline = {
			type = "group",
			order = 1,
			name = " ",
			inline = true,
			args = {
				speed = {
					type = "range",
					order = 1,
					width = "double",
					descStyle = "",
					name = L["Camera Zoom Increment"],
					get = function(i) return db.increment end,
					set = function(i, v) db.increment = v end,
					min = 1,
					max = 50, softMax = 25,
					step = 1,
				},
				newline1 = {type = "description", order = 2, name = "\n"},
				cameraDistanceMoveSpeed = {
					type = "range",
					order = 3,
					width = "double",
					descStyle  = "",
					name = L["Camera Zoom Speed"],
					get = function(i) return tonumber(GetCVar("cameraDistanceMoveSpeed")) end,
					set = function(i, v) SetCVar("cameraDistanceMoveSpeed", v); db.speed = v end,
					min = 1,
					max = 50,
					step = 1,
				},
				newline2 = {type = "description", order = 4, name = "\n"},
				cameraDistanceMax = {
					type = "range",
					order = 5,
					width = "double",
					descStyle = "",
					name = MAX_FOLLOW_DIST,
					get = function(i) return tonumber(GetCVar("cameraDistanceMax")) end,
					set = function(i, v) SetCVar("cameraDistanceMax", v); db.distance = v end,
					min = 1,
					max = 50,
					step = 1,
				},
			},
		},
		spacing1 = {type = "description", order = 2, name = "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"},
		reset = {
			type = "execute",
			order = 3,
			descStyle = "",
			name = RESET,
			confirm = true, confirmText = RESET.."?",
			func = function()
				db.increment = 5
				db.speed = tonumber(GetCVarDefault("cameraDistanceMoveSpeed"))
				db.distance = tonumber(GetCVarDefault("cameraDistanceMax"))
				for _, v in ipairs(cvar) do
					SetCVar(v, GetCVarDefault(v))
				end
			end,
		},
	},
}

	----------------------
	--- Initialization ---
	----------------------

local delay = 0
local noop = function() end

-- delay setting/overriding the CVars because it's either
-- not yet ready or is being reverted/overridden by something else
local function OnUpdate(self, elapsed)
	delay = delay + elapsed
	if delay > 1 then
		SetCVar("cameraDistanceMoveSpeed", db.speed)
		SetCVar("cameraDistanceMax", db.distance)
		-- cameraDistanceMax should be the only thing controlling
		-- the max camera distance, for accuracy and simplicity's sake
		SetCVar("cameraDistanceMaxFactor", 1)
		-- Disable the Blizzard Camera Slider to "hint" users it's being replaced or overridden
		InterfaceOptionsCameraPanelMaxDistanceSlider:Disable()
		self:SetScript("OnUpdate", noop)
	end
end

local defaults = {
	db_version = 0.9, -- update this on (major) savedvars changes
	increment = 5,
	speed = 20,
	distance = 50,
}

local optionsFrame

local function OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and ... == NAME then
		if not FasterCameraDB2 or FasterCameraDB2.db_version ~= 0.9 then
			FasterCameraDB2 = defaults
		end
		db = FasterCameraDB2
		LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("FasterCamera", options)
		optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FasterCamera", NAME)
		self:SetScript("OnUpdate", OnUpdate)
		self:UnregisterEvent("ADDON_LOADED")
	end
end

local f = CreateFrame("Frame")
f:RegisterEvent("ADDON_LOADED")
f:SetScript("OnEvent", OnEvent)

	---------------------
	--- Slash Command ---
	---------------------

SLASH_FASTERCAMERA1 = "/fc"
SLASH_FASTERCAMERA2 = "/fastercam"
SLASH_FASTERCAMERA3 = "/fastercamera"

SlashCmdList["FASTERCAMERA"] = function(msg, editbox)
	InterfaceOptionsFrame_OpenToCategory(optionsFrame)
end

	---------------------
	--- LibDataBroker ---
	---------------------

local function distanceString()
	local maxDistance = GetCVar("cameraDistanceMax") * GetCVar("cameraDistanceMaxFactor")
	return maxDistance<=50 and maxDistance or 50
end

local dataobject = {
	type = "launcher",
	icon = "Interface\\Icons\\Spell_ChargePositive",
	text = NAME,
	OnClick = function(clickedframe, button)
		if InterfaceOptionsFrame:IsShown() and InterfaceOptionsFramePanelContainer.displayedPanel.name == NAME then
			InterfaceOptionsFrame:Hide()
		else
			InterfaceOptionsFrame_OpenToCategory(optionsFrame)
		end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine("|cffADFF2F"..NAME.."|r")
		tt:AddDoubleLine(L["Camera Zoom Increment"], "|cffFFFFFF"..db.increment.."|r")
		tt:AddDoubleLine(L["Camera Zoom Speed"], "|cffFFFFFF"..db.speed.."|r")
		tt:AddDoubleLine(MAX_FOLLOW_DIST, "|cffFFFFFF"..distanceString().."|r")
	end,
}

LibStub("LibDataBroker-1.1"):NewDataObject("FasterCamera", dataobject)